/*
 *                            *
 *                           * *
 *                          *   *
 *                     ***************
 *                      * *       * *
 *                       *  MUMPS  *
 *                      * *       * *
 *                     ***************
 *                          *   *
 *                           * *
 *                            *
 *
 *   iniconf.c
 *    Function implementations for reading
 *    FreeM configuration files
 *
 *  
 *   Author: John P. Willis <jpw@coherent-logic.com>
 *    Copyright (C) 1998 MUG Deutschland
 *    Copyright (C) 2020 Coherent Logic Development LLC
 *
 *
 *   This file is part of FreeM.
 *
 *   FreeM is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Affero Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   FreeM is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Affero Public License for more details.
 *
 *   You should have received a copy of the GNU Affero Public License
 *   along with FreeM.  If not, see <https://www.gnu.org/licenses/>.
 *
 **/

#define FALSE 0
#define TRUE 1
#include <stddef.h>
#include "iniconf.h"
#include "libfill.h"
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>
#include <string.h>
#include <limits.h>

extern char config_file[4096];

#if !defined(PATH_MAX) && defined(_SCO_DS)
# define PATH_MAX 4096
#endif

#if !defined(PATH_MAX) && defined(__gnu_hurd__)
# define PATH_MAX 1024
#endif

#if !defined(PATH_MAX) && defined(_AIX)
# define PATH_MAX 1024
#endif

#if defined(__NetBSD__) || defined(__FreeBSD__)
# include <sys/syslimits.h>
#endif

/* prototypes for internal-use-only functions */
int read_profile_string(char *file, char *section, char *key, char *value);

int get_conf(char *section, char *key, char *value)
{
    char *etcfile;
    char *dotfile;
    char *homedir;

    int exists_in_etc = FALSE;
    int exists_in_dotfile = FALSE;
    int dotexists;
    int etcexists;

    char *etc_value = (char *) malloc(CONF_BUFSIZE);
    char *dot_value = (char *) malloc(CONF_BUFSIZE);

    etcfile = config_file;


#if !defined(__AMIGA) && !defined(EMSCRIPTEN) && !defined(_SCO_DS) && !defined(_AIX)
    uid_t uid = geteuid();
    struct passwd *pw = getpwuid(uid);

    if (pw == NULL) return(FALSE);

    homedir = (char *) calloc(strlen(pw->pw_dir) + 1, sizeof(char));
    (void) strcpy(homedir, pw->pw_dir);

    dotfile = calloc(PATH_MAX, sizeof(char));

    (void) strcat(dotfile, homedir);
    (void) strcat(dotfile, "/.freemrc");

    etcexists = file_exists(etcfile);
    dotexists = file_exists(dotfile);
#else

#if defined(__AMIGA)
    strcpy (etcfile, "./freem.conf");
    etcexists = TRUE;
    dotexists = FALSE;
#else
    strcpy (etcfile, SYSCONFDIR"/freem.conf");
    etcexists = TRUE;
    dotexists = FALSE;
#endif

#endif

    if (etcexists == TRUE) {
        exists_in_etc = read_profile_string(etcfile, section, key, etc_value);
    }
    else {
        exists_in_etc = FALSE;
    }

    if (dotexists == TRUE) {
        exists_in_dotfile = read_profile_string(dotfile, section, key, dot_value);
    }
    else {
        exists_in_dotfile = FALSE;
    }

    if (exists_in_dotfile) {
        strcpy(value, dot_value);
        return(TRUE);
    }

    if (exists_in_etc) {
        strcpy(value, etc_value);
        return(TRUE);
    }  

    return(FALSE); /* didn't exist anywhere */
}

int read_profile_string(char *file, char *section, char *key, char *value)
{

    register int i;

    FILE *fp;

    char *fullsec = (char *) malloc(CONF_BUFSIZE);
    char *curkey = (char *) malloc(CONF_BUFSIZE);
    char *curval = (char *) malloc(CONF_BUFSIZE);
    char *cursec = (char *) malloc(CONF_BUFSIZE);
    char *line = (char *) malloc(CONF_BUFSIZE);
    int lnum = 0;


    snprintf(fullsec, CONF_BUFSIZE, "[%s]%c", section, '\0');

    strcpy(cursec, "[]");


    fp = fopen(file, "r");

    while(fgets(line, CONF_BUFSIZE, fp) != NULL) {
        ++lnum;

        if(line[0] == '[') {
            strcpy(cursec, line);

            for(i = 0; i < CONF_BUFSIZE; i++) {
                if(cursec[i] == ']') {
                    cursec[i + 1] = '\0';
                    break;
                }
            }
        }
        else {
            if ((line[0] != '[') && (strchr(line, '=') != NULL)) {
                curkey = strtok(line, "=");
                curval = strtok(NULL, "=");  
                curval = strtok(curval, "\n");
       

                if((strcmp(curkey, key) == 0) && (strcmp(cursec, fullsec) == 0)) {                                                
                    strcpy(value, curval);
                    (void) fclose(fp);
                    return(TRUE);
                }
            }
        }        

    }

    if (fp != NULL) {
        (void) fclose(fp);
    }

    /* if we've gotten here, the section and/or key was not found */
    return(FALSE);

}

int file_exists(char *filename)
{
    FILE *fp;

    if ((fp = fopen(filename, "r")) != NULL) {
        (void) fclose(fp);
        
        return(TRUE);
    }
    else {
        return(FALSE);
    } 
}
